# coding: utf-8

# Copyright 2023 IBM All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio

from ibm_watson_openscale.utils.utils import check_package_exists


def run_in_event_loop(task):
    """Run the given task in async event loop until complete"""
    
    event_loop = asyncio.get_event_loop()
    if event_loop.is_closed():
        asyncio.set_event_loop(asyncio.new_event_loop())
        event_loop = asyncio.get_event_loop()

    if event_loop.is_running():
        # If the event loop is running e.g. in a jupyter notebook
        # This nest_asyncio will patch the current event loop
        # to accept a new task.
        # Not adding this nest_asyncio as dependency as this 
        # is already bundled with Jupyter
        check_package_exists("nest_asyncio")
        import nest_asyncio
        nest_asyncio.apply()

    result = event_loop.run_until_complete(task)

    if not event_loop.is_running():
        event_loop.close()
    return result